/*******************************************************************************
  MRF24WG Configuration Data

  File Name:
    drv_wifi_config_data.c

  Summary:
    MRF24WG Configuration Data

  Description:
    - Stores and retrieves Wi-Fi configuration information to non-volatile
      memory (NVM)
 *******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/

#include <string.h>
#include "tcpip/src/tcpip_private.h"

#define WF_NVM_SPACE_MAGIC_NUMBER 0x5a5a5a5a

DRV_WIFI_CONFIG_DATA wifi_ConfigData;
DRV_WIFI_CONFIG_DATA *p_wifi_configData = &wifi_ConfigData;

#if defined(DRV_WIFI_NVM_SPACE_ENABLE)
uint8_t __attribute__((coherent)) NVM_DATA_BUFF_local[DRV_NVM_ROW_SIZE];

SYS_FS_MEDIA_GEOMETRY *gAppNVMMediaGeometry;

bool DRV_WIFI_NVM_write(uint8_t *buf, int size)
{
    int i, tmp;
    DRV_HANDLE nvmHandle;
    DRV_NVM_COMMAND_HANDLE nvmCommandHandle;
    uint8_t NVM_DATA_READ_BUFF_local[DRV_NVM_ROW_SIZE];

    if (size > DRV_NVM_ROW_SIZE) {
        return false;
    }

    for (i = 0; i < size; i++)
        NVM_DATA_BUFF_local[i] = buf[i];

    nvmHandle = DRV_NVM_Open(0, DRV_IO_INTENT_READWRITE);
    if (DRV_HANDLE_INVALID == nvmHandle) {
        return false;
    }

    gAppNVMMediaGeometry = DRV_NVM_GeometryGet(nvmHandle);
    if (NULL == gAppNVMMediaGeometry) {
        return false;
    }

    DRV_NVM_Read (nvmHandle, &nvmCommandHandle, NVM_DATA_READ_BUFF_local, DRV_WIFI_NVM_SPACE_ADDR, DRV_NVM_ROW_SIZE);
    if (DRV_NVM_COMMAND_HANDLE_INVALID == nvmCommandHandle) {
        return false;;
    }

    tmp = DRV_WIFI_NVM_SPACE_ADDR * (gAppNVMMediaGeometry->geometryTable[2].numBlocks) / (DRV_NVM_MEDIA_SIZE * 1024);
    DRV_NVM_Erase(nvmHandle, &nvmCommandHandle,  tmp, 1);
    if (nvmCommandHandle == DRV_NVM_COMMAND_HANDLE_INVALID) {
        return false;;
    }

    while (DRV_NVM_COMMAND_COMPLETED != DRV_NVM_CommandStatus(nvmHandle, nvmCommandHandle))
        ;

    tmp = DRV_WIFI_NVM_SPACE_ADDR * (gAppNVMMediaGeometry->geometryTable[1].numBlocks) / (DRV_NVM_MEDIA_SIZE * 1024);
    DRV_NVM_Write(nvmHandle, &nvmCommandHandle, (uint8_t *)NVM_DATA_BUFF_local, tmp, 1);
    if (DRV_NVM_COMMAND_HANDLE_INVALID == nvmCommandHandle) {
        return false;;
    }

    while (DRV_NVM_COMMAND_COMPLETED != DRV_NVM_CommandStatus(nvmHandle, nvmCommandHandle))
        ;

    DRV_NVM_Close(nvmHandle);

    return true;
}

bool DRV_WIFI_NVM_read(uint8_t *buf, int size)
{
    DRV_HANDLE nvmHandle;
    DRV_NVM_COMMAND_HANDLE nvmCommandHandle;

    if (size > DRV_NVM_PAGE_SIZE) {
        return false;
    }

    nvmHandle = DRV_NVM_Open(0, DRV_IO_INTENT_READWRITE);
    if (DRV_HANDLE_INVALID == nvmHandle) {
        return false;
    }

    gAppNVMMediaGeometry = DRV_NVM_GeometryGet(nvmHandle);
    if (NULL == gAppNVMMediaGeometry) {
        return false;
    }

    DRV_NVM_Read(nvmHandle, &nvmCommandHandle, buf, DRV_WIFI_NVM_SPACE_ADDR, size);
    if (DRV_NVM_COMMAND_HANDLE_INVALID == nvmCommandHandle) {
        return false;
    }

    while (DRV_NVM_COMMAND_COMPLETED != DRV_NVM_CommandStatus(nvmHandle, nvmCommandHandle))
        ;

    DRV_NVM_Close(nvmHandle);

    return true;
}

bool DRV_WIFI_NVM_erase(void)
{
    int tmp;
    DRV_HANDLE nvmHandle;
    DRV_NVM_COMMAND_HANDLE nvmCommandHandle;
    uint8_t NVM_DATA_READ_BUFF_local[DRV_NVM_ROW_SIZE];

    nvmHandle = DRV_NVM_Open(0, DRV_IO_INTENT_READWRITE);
    if (DRV_HANDLE_INVALID == nvmHandle) {
        return false;
    }

    gAppNVMMediaGeometry = DRV_NVM_GeometryGet(nvmHandle);
    if (NULL == gAppNVMMediaGeometry) {
        return false;
    }

    DRV_NVM_Read(nvmHandle, &nvmCommandHandle, NVM_DATA_READ_BUFF_local, DRV_WIFI_NVM_SPACE_ADDR, DRV_NVM_ROW_SIZE);
    if (DRV_NVM_COMMAND_HANDLE_INVALID == nvmCommandHandle) {
        return false;;
    }

    tmp = DRV_WIFI_NVM_SPACE_ADDR * (gAppNVMMediaGeometry->geometryTable[2].numBlocks) / (DRV_NVM_MEDIA_SIZE * 1024);
    DRV_NVM_Erase(nvmHandle, &nvmCommandHandle,  tmp, 1);
    if (nvmCommandHandle == DRV_NVM_COMMAND_HANDLE_INVALID) {
        return false;;
    }

    while (DRV_NVM_COMMAND_COMPLETED != DRV_NVM_CommandStatus(nvmHandle, nvmCommandHandle))
        ;

    DRV_NVM_Close(nvmHandle);

    return true;
}

void WriteConfigToMemory(void)
{
    WF_CONFIG_PARAMS(verify_flag) = WF_NVM_SPACE_MAGIC_NUMBER;
    DRV_WIFI_NVM_write((uint8_t *)p_wifi_configData, sizeof(DRV_WIFI_CONFIG_DATA));
}

void ReadConfigFromMemory(void)
{
    DRV_WIFI_NVM_read((uint8_t *)p_wifi_configData, sizeof(DRV_WIFI_CONFIG_DATA));
}

void EraseConfigFromMemory(void)
{
    DRV_WIFI_NVM_erase();
}

#else // !defined(DRV_WIFI_NVM_SPACE_ENABLE)

DRV_WIFI_CONFIG_DATA SaveWiFiConf_in_memory = {0};

bool DRV_WIFI_NVM_write(uint8_t *buf, int size) {return true;}
bool DRV_WIFI_NVM_read(uint8_t *buf, int size) {return true;}
bool DRV_WIFI_NVM_erase(void) {return true;}

void WriteConfigToMemory(void)
{
    int i;
    uint8_t *p_data1, *p_data2;

    WF_CONFIG_PARAMS(verify_flag) = WF_NVM_SPACE_MAGIC_NUMBER;
    p_data1 = (void *)p_wifi_configData;
    p_data2 = (void *)&SaveWiFiConf_in_memory;
    for (i = 0; i < sizeof(DRV_WIFI_CONFIG_DATA); i++)
    {
        p_data2[i] = p_data1[i];
    }
}

void ReadConfigFromMemory(void)
{
    int i;
    uint8_t *p_data1, *p_data2;

    p_data1 = (void *)&SaveWiFiConf_in_memory;
    p_data2 = (void *)p_wifi_configData;
    for (i = 0; i < sizeof(DRV_WIFI_CONFIG_DATA); i++)
    {
        p_data2[i] = p_data1[i];
    }
}

void EraseConfigFromMemory(void)
{
    memset(&SaveWiFiConf_in_memory, 0, sizeof(SaveWiFiConf_in_memory));
}

#endif // #if defined(DRV_WIFI_NVM_SPACE_ENABLE)

bool DRV_WIFI_NvmVerify(void)
{
    uint32_t Space_verified = 0x00;

    DRV_WIFI_NVM_read((uint8_t *)&Space_verified, sizeof(Space_verified));
    
    return (Space_verified == WF_NVM_SPACE_MAGIC_NUMBER);
}

#if defined(TCPIP_STACK_COMMANDS_WIFI_ENABLE)
/*******************************************************************************
  Function:
    void DRV_WIFI_ConfigDataPrint(void)

  Summary:
    Outputs to console the configuration data from the board EEPROM.

  Description:
    This function outputs configuration data from the board EEPROM.

  Precondition:
    TCPIP stack should be initialized.

  Parameters:
    None.

  Returns:
    None.

  Remarks:
    None.
 *******************************************************************************/
void DRV_WIFI_ConfigDataPrint(void)
{
    int i;
    uint8_t tmpNetSSID[DRV_WIFI_MAX_SSID_LENGTH + 1] = {0};
    DRV_WIFI_CONFIG_DATA tmp_wifi_configData;

    DRV_WIFI_NVM_read((uint8_t *)&tmp_wifi_configData, sizeof(DRV_WIFI_CONFIG_DATA));

    if (WF_NVM_SPACE_MAGIC_NUMBER != tmp_wifi_configData.verify_flag) {
        SYS_CONSOLE_MESSAGE("!!Cannot read data from storage space, because it is not initialized\r\n");
        return;
    }

    SYS_CONSOLE_PRINT("\r\nSize of Configdata:  %lu\r\n", sizeof(tmp_wifi_configData));

    for (i = 0; i < tmp_wifi_configData.SsidLength; i++) {
        tmpNetSSID[i] = tmp_wifi_configData.netSSID[i];
    }

    SYS_CONSOLE_PRINT("networkType:         %d\r\n", tmp_wifi_configData.networkType);
    SYS_CONSOLE_PRINT("netSSID:             %s\r\n", tmpNetSSID);
    SYS_CONSOLE_PRINT("SsidLength:          %d\r\n", tmp_wifi_configData.SsidLength);
    SYS_CONSOLE_PRINT("SecurityMode:        %d\r\n", tmp_wifi_configData.SecurityMode);
    SYS_CONSOLE_MESSAGE("SecurityKey:         ");
    for (i = 0; i < tmp_wifi_configData.SecurityKeyLength; i++) {
        SYS_CONSOLE_PRINT("%.2x", tmp_wifi_configData.SecurityKey[i]);
    }

    SYS_CONSOLE_MESSAGE("\r\n");
    SYS_CONSOLE_PRINT("SecurityKeyLength:   %d\r\n", tmp_wifi_configData.SecurityKeyLength);
}
#endif /* TCPIP_STACK_COMMANDS_WIFI_ENABLE */

/*******************************************************************************
  Function:
    bool DRV_WIFI_ConfigDataErase(void)

  Summary:
    Erases configuration data from the board EEPROM.

  Description:
    This function erases configuration data from the board EEPROM.

 Parameters:
    None.

  Returns:
    None
 *******************************************************************************/
bool DRV_WIFI_ConfigDataErase(void)
{
    EraseConfigFromMemory();

    return true;
}

//DOM-IGNORE-END
