/*******************************************************************************
  MRF24WG RTOS Wrapper Functions

  File Name:
    drv_wifi_rtos_wrapper.c

  Summary:
    MRF24WG RTOS Wrapper Functions

  Description:
    RTOS wrapper functions for MRF24WG Wi-Fi module.
 *******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/

#include "tcpip/src/tcpip_private.h"

#if defined(DRV_WIFI_USE_FREERTOS)
#include "drv_wifi_rtos_wrapper.h"
#include "drv_wifi_scan_helper.h"

bool DRV_WIFI_MutexInit(OSAL_MUTEX_HANDLE_TYPE *mutex)
{
    if (OSAL_MUTEX_Create(mutex) == OSAL_RESULT_TRUE)
        return true;
    else
        return false;
}

void DRV_WIFI_MutexDeInit(OSAL_MUTEX_HANDLE_TYPE *mutex)
{
    OSAL_MUTEX_Delete(mutex);
}

void DRV_WIFI_MutexLock(OSAL_MUTEX_HANDLE_TYPE *mutex, uint16_t waitMS)
{
    OSAL_MUTEX_Lock(mutex, waitMS);
}

void DRV_WIFI_MutexUnlock(OSAL_MUTEX_HANDLE_TYPE *mutex)
{
    OSAL_MUTEX_Unlock(mutex);
}

bool DRV_WIFI_SemInit(OSAL_SEM_HANDLE_TYPE *sem)
{
    if (OSAL_SEM_Create(sem, OSAL_SEM_TYPE_COUNTING, 1, 0) == OSAL_RESULT_TRUE)
        return true;
    else
        return false;
}

void DRV_WIFI_SemDeInit(OSAL_SEM_HANDLE_TYPE *sem)
{
    OSAL_SEM_Delete(sem);
}

void DRV_WIFI_SemTake(OSAL_SEM_HANDLE_TYPE *sem, uint16_t waitMS)
{
    OSAL_SEM_Pend(sem, waitMS);
}

void DRV_WIFI_SemGive(OSAL_SEM_HANDLE_TYPE *sem)
{
    OSAL_SEM_Post(sem);
}

void DRV_WIFI_ISR_SemGive(OSAL_SEM_HANDLE_TYPE *sem)
{
    OSAL_SEM_PostISR(sem);
}

void DRV_WIFI_DeferredISR_SemGive(void)
{
    DRV_WIFI_ISR_SemGive(&g_drv_wifi_priv.deferredISRSync);
}

bool DRV_WIFI_TaskSyncInit(void)
{
    static bool once = false;
    
    if (!once) {     
        if (!DRV_WIFI_MutexInit(&g_drv_wifi_priv.spiLock))
            return false;

        if (!DRV_WIFI_MutexInit(&g_drv_wifi_priv.multicastFilterLock))
            return false;

        if (!DRV_WIFI_MutexInit(&g_drv_wifi_priv.mgmtMsgLock))
            return false;

        if (!DRV_WIFI_SemInit(&g_drv_wifi_priv.macTaskSync))
            return false;

        if (!DRV_WIFI_SemInit(&g_drv_wifi_priv.deferredISRSync))
            return false;

        if (!DRV_WIFI_SemInit(&g_drv_wifi_priv.mgmtRespSync))
            return false;

        if (!DRV_WIFI_SemInit(&g_drv_wifi_priv.initSync))
            return false;

        once = true;
    }
    
    return true;
}

void DRV_WIFI_TaskDestroy(TaskHandle_t task_handle)
{
    vTaskDelete(task_handle);
}

#endif /* defined(DRV_WIFI_USE_FREERTOS) */

//DOM-IGNORE-END
