/*******************************************************************************
  TCP/IP MRF24WN Scan Helper Implementation

  File Name:
    wdrv_mrf24wn_scan_helper.c

  Summary:
    Module for Microchip TCP/IP Stack

  Description:
    -Provides access to MRF24WN WiFi controller
    -Reference: MRF24WN Datasheet, IEEE 802.11 Standard
 *******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/

#include <ctype.h>

#include "wdrv_mrf24wn_main.h"
#include "wdrv_mrf24wn_scan_helper.h"

WDRV_SCAN_RESULT g_scanResults[30];

uint16_t WDRV_PreScanStart(void)
{
    /* If scan already in progress bail out */
    if (IS_SCAN_IN_PROGRESS(g_scanStatus.scanState))
    {
        return 0;
    }
    
    WDRV_EXT_CmdScanStart();

    SCAN_SET_IN_PROGRESS(g_scanStatus.scanState);

    return WDRV_SUCCESS;
}

void WDRV_ScanResultsDisplayManager(void)
{
    int i;
    char ssid[32 + 1];
    static bool displayHeader = true;
    WDRV_SCAN_RESULT scanResult;

    if ((g_scanStatus.numScanResults == 0) ||
        (!IS_SCAN_STATE_DISPLAY(g_scanStatus.scanState)) ||
        (IS_SCAN_IN_PROGRESS(g_scanStatus.scanState)) ||
        (!IS_SCAN_STATE_VALID(g_scanStatus.scanState)))
    {
       SCAN_CLEAR_DISPLAY(g_scanStatus.scanState);
       return;
    }

    if (displayHeader) {
        g_scanStatus.displayIdx = 0;
        SYS_CONSOLE_MESSAGE((const char *)"\nWi-Fi scan results ... \r\n\n");
        if (g_scanStatus.numScanResults == 0) {
            /* No AP found, re-initialize the scan context state. Scan is not allowed in Soft AP mode.
             * However, g_scanStatus.numScanResults needs to be preserved for webpage using.
             */
            SYS_CONSOLE_MESSAGE((const char *)"No AP found.\r\n ");
            SCAN_CLEAR_DISPLAY(g_scanStatus.scanState);
            if (WDRV_CONFIG_PARAMS(networkType) == WDRV_NETWORK_TYPE_SOFT_AP) {
                SCAN_CLEAR_VALID(g_scanStatus.scanState);
                return;
            } else {
                return;
            }
        } else {
            SYS_CONSOLE_MESSAGE((const char *)"     SSID                             RSSI  Channel\r\n");
            SYS_CONSOLE_MESSAGE((const char *)"     -------------------------------- ----  -------\r\n");
            displayHeader = false;
        }
    }
    
    WDRV_EXT_ScanGetResult(g_scanStatus.displayIdx, &scanResult);

    memset(ssid, ' ', sizeof(ssid));

    /* Display SSID */
    for (i = 0; i < scanResult.ssidLen; i++)
    {
        if (!isprint(scanResult.ssid[i]))
        {
            ssid[i] = '*';
        }
        else
        {
            ssid[i] = scanResult.ssid[i];
        }
    }

    ssid[32] = 0;

    /* Display Index & SSID */
    //SYS_CONSOLE_PRINT("%2d)  ", g_scanStatus.displayIdx + 1);
    //SYS_CONSOLE_PRINT("%s", ssid);

    /* Display RSSI & Channel */
    /* RSSI_MAX : 128, RSSI_MIN : 43 */
    //SYS_CONSOLE_PRINT(" %2u    %u\r\n", scanResult.rssi, scanResult.channel);

    SYS_CONSOLE_PRINT(" %2d) %s %2u    %u\r\n", g_scanStatus.displayIdx + 1, ssid, scanResult.rssi, scanResult.channel);

    if (WDRV_CONFIG_PARAMS(networkType) == WDRV_NETWORK_TYPE_SOFT_AP) // WF_PRESCAN
    {
        g_scanResults[g_scanStatus.displayIdx] = scanResult; // WF_PRESCAN
        if (g_scanStatus.displayIdx == sizeof(g_scanResults) / sizeof(g_scanResults[0]) - 1)
        {
            /* Prescan is finished, re-initialize the scan context state. Scan is not allowed in Soft AP mode.
             * However, g_scanStatus.numScanResults needs to be preserved for webpage using.
             */
            SCAN_CLEAR_VALID(g_scanStatus.scanState);
            SCAN_CLEAR_DISPLAY(g_scanStatus.scanState);
            g_scanStatus.displayIdx = 0;
            displayHeader = true;
        }
    }

    if (++g_scanStatus.displayIdx == g_scanStatus.numScanResults) // Normal scan which does not happen before Wi-Fi connect
    {
        if (WDRV_CONFIG_PARAMS(networkType) == WDRV_NETWORK_TYPE_SOFT_AP)
        {
            /* Scan is finished, re-initialize the scan context state. Scan is not allowed in Soft AP mode.
             * However, g_scanStatus.numScanResults needs to be preserved for webpage using.
             */
            SCAN_CLEAR_VALID(g_scanStatus.scanState);
            SCAN_CLEAR_DISPLAY(g_scanStatus.scanState);
            g_scanStatus.displayIdx = 0;
            displayHeader = true;
        }
        else
        {
            SCAN_CLEAR_DISPLAY(g_scanStatus.scanState);
            g_scanStatus.displayIdx = 0;
            displayHeader = true;
        }
    }
}

//DOM-IGNORE-END
